#include <stdlib.h>

#include "configuration.h"

ConfigFile                    * Configuration::configFile = NULL;

Configuration::Configuration(void)
{
}

status_t Configuration::init(void)
{
  if (configFile) return T_OK;

  QDir dir(getSettingsRoot());
  if (!dir.isReadable())
    dir.mkdir(getSettingsRoot(), true);

  configFile = new ConfigFile(dir.path() + "/mpdc.ini");

  return T_OK;
}

status_t Configuration::close(void)
{
  if (configFile)
    delete configFile;

  return T_OK;
}

// Get functions

QString Configuration::getSettingsRoot(void)
{
  const char *hd = getenv("APPDATA");

  if (hd)
  {
    QString homedir = hd;
    if (homedir.length() < 5)
      homedir = QDir::cleanDirPath(QDir::homeDirPath());

    return QDir::cleanDirPath(homedir + "/");
  }

  return QDir::cleanDirPath(QDir::homeDirPath());
}

QRect Configuration::getGeometry(QString windowname)
{
  if (init() < T_OK) return QRect(205,25,400,100);

  configFile->setGroup("Geometry");
  QStringList s = configFile->readListEntry(windowname, ',');

  if (s.count() != 4)
    return QRect(205,25,400,100);

  QStringList::Iterator it = s.begin();

  QRect rect;
  rect.setLeft  ( (*it++).toInt() );
  rect.setTop   ( (*it++).toInt() );
  rect.setRight ( (*it++).toInt() );
  rect.setBottom( (*it++).toInt() );

  return rect;
}

bool Configuration::getPlaylistVisible(void)
{
  if (init() < T_OK) return false;

  configFile->setGroup("Settings");
  return configFile->readBoolEntry("playlistvisible", false);
}

bool Configuration::getCountMode(void)
{
  if (init() < T_OK) return false;

  configFile->setGroup("Settings");
  return configFile->readBoolEntry("countmode", false);
}

QString Configuration::getSkin(void)
{
  if (init() < T_OK) return "";

  configFile->setGroup("Settings");
  return configFile->readEntry("skin", "");
}

QString Configuration::getServerIP(void)
{
  if (init() < T_OK) return "";

  configFile->setGroup("Server");
  return configFile->readEntry("address", "10.0.0.240");
}

unsigned short Configuration::getServerPort(void)
{
  if (init() < T_OK) return 2100;

  configFile->setGroup("Server");
  return configFile->readNumEntry("port", 2100);
}

QString Configuration::getSkinDir(void)
{
  if (init() < T_OK) return "";

  configFile->setGroup("Settings");
  #ifdef WIN32
    return configFile->readEntry("skindir", "./skins/");
  #else
    return configFile->readEntry("skindir", QDir::cleanDirPath(QDir::homeDirPath() + "/waskins/"));
  #endif
}

// Set functions

status_t Configuration::setGeometry(QString windowname, QRect geo)
{
  if (init() < T_OK) return T_ERROR;

  QString s  = QString::number(geo.left()) + ", ";
          s += QString::number(geo.top()) + ", ";
          s += QString::number(geo.right()) + ", ";
          s += QString::number(geo.bottom());

  configFile->setGroup("Geometry");
  configFile->writeEntry(windowname, s);

  if (configFile->write() < T_OK) return T_ERROR;
  return T_OK;
}

status_t Configuration::setPlaylistVisible(bool visible)
{
  if (init() < T_OK) return T_ERROR;

  configFile->setGroup("Settings");
  configFile->writeEntry("playlistvisible", visible);

  if (configFile->write() < T_OK) return T_ERROR;
  return T_OK;
}

status_t Configuration::setCountMode(bool countMode)
{
  if (init() < T_OK) return T_ERROR;

  configFile->setGroup("Settings");
  configFile->writeEntry("countmode", countMode);

  if (configFile->write() < T_OK) return T_ERROR;
  return T_OK;
}

status_t Configuration::setSkin(QString skin)
{
  if (init() < T_OK) return T_ERROR;

  configFile->setGroup("Settings");
  configFile->writeEntry("skin", skin);

  if (configFile->write() < T_OK) return T_ERROR;
  return T_OK;
}

status_t Configuration::setServerIP(QString address)
{
  if (init() < T_OK) return T_ERROR;

  configFile->setGroup("Server");
  configFile->writeEntry("address", address);

  if (configFile->write() < T_OK) return T_ERROR;
  return T_OK;
}

status_t Configuration::setServerPort(unsigned short port)
{
  if (init() < T_OK) return T_ERROR;

  std::cout << "Port: " << port << std::endl;

  configFile->setGroup("Server");
  configFile->writeEntry("port", port);

  if (configFile->write() < T_OK) return T_ERROR;
  return T_OK;
}

status_t Configuration::setSkinDir(QString dir)
{
  if (init() < T_OK) return T_ERROR;

  configFile->setGroup("Settings");
  configFile->writeEntry("skindir", dir);

  if (configFile->write() < T_OK) return T_ERROR;
  return T_OK;
}

