#ifndef __CONTROLPANEL__
#define __CONTROLPANEL__

#include <qpushbutton.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qlcdnumber.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qslider.h>
#include <qframe.h>
#include <qdir.h>
#include <qpopupmenu.h>
#include <vector>

#include "mpdcom.h"
#include "playlistwindow.h"
#include "configdialog.h"
#include "waskin.h"
#include "gbutton.h"
#include "gtextview.h"
#include "gslider.h"

/*! \author     <a href="mailto:mpdc@hokitoki.dds.nl">ing. A.E. van Putten</a>
    \author     <a href="mailto:a.j.admiraal@cs.utwente.nl">ing. A.J. Admiraal</a>
    \date       01-01-2004
    \brief      ControlPanel class for status display and controlling the Music Player Daemon.
 */
class ControlPanel : public QWidget
{
Q_OBJECT
private:
  class Display : public QLCDNumber
  {
  public:
    inline                      Display(QWidget *parent) : QLCDNumber(parent), reverseCounter(false) { }

  protected:
    virtual void                mousePressEvent(QMouseEvent *);

  public:
    bool                        reverseCounter;
  };

public:
                                ControlPanel( QWidget *parent=0, const char *name=0, WFlags f=0 );

  void                          setSkin(WASkin *);
  void                          playListClosed(void);

protected slots:
  void                          slotUpdateDisplay(void);                        //!< Refreshes the displayed information, triggered by a timer
  void                          slotReceiveUpdate(const MpdCom::Status &);      //!< Refreshes the displayed information and synchronises the timer, triggered by the communication class
  void                          slotLockVolume(void);                           //!< Locks the volume slider from status updates when it's used
  void                          slotReverseVolume(int);                         //!< Reverses the volume slider
  void                          slotReleaseVolume(void);                        //!< Releases the lock on the volume
  void                          slotLockBalance(void);                          //!< Locks the balance slider from status updates when it's used
  void                          slotReverseBalance(int);                        //!< Reverses the balance slider
  void                          slotReleaseBalance(void);                       //!< Releases the lock on the volume
  void                          slotLockSeek(void);                             //!< Locks the seek slider from status updates when it's used
  void                          slotSetSeek(int);                               //!< Sets the disired seek position
  void                          slotReleaseSeek(void);                          //!< Releases the seek slider
  void                          slotShowPlaylist(bool);                         //!< Shows/hides the playlist
  void                          slotShowSettings(void);                         //!< Shows the settings
  void                          slotError(const QString &);                     //!< Receives errors from the communication class
  void                          slotAbout(void);                                //!< Shows the aboutbox
  void                          slotOpenSkin(void);                             //!< Installs the preferred skin.
  void                          slotRebuildSkinMenu(void);                      //!< Rebuilds the skin menu.
  void                          slotEjectPressed(bool);                         //!< Shows the file manager.

protected:
  virtual void                  closeEvent(QCloseEvent *);                      //!< Stores it's window position and closes
  virtual void                  paintEvent(QPaintEvent *);
  virtual void                  mouseMoveEvent(QMouseEvent *);
  virtual void                  mousePressEvent(QMouseEvent *);
  virtual void                  mouseReleaseEvent(QMouseEvent *);
  virtual void                  keyPressEvent(QKeyEvent *);

private:
  WASkin                        defaultSkin;
  GButton                       btPrev, btPlay, btPause, btStop, btNext, btEject;
  GButton                       btEq, btPl, btShuffle, btRepeat;
  GTextView                     txTitle, txKbps, txKHz;
  GSlider                       slVolume, slBalance;
  GSlider                       slPosition;
  GButton                       btMin, btMax, btClose;

  QTimer                        timer, dirTimer;
  QDir                          dir;
  QPopupMenu                    mainMenu;
  QPopupMenu                    skinMenu;
  std::vector<WASkin *>         skins;
  bool                          countMode;
  bool                          volumeEdit;
  bool                          balanceEdit;
  bool                          seekEdit;
  bool                          mouseDown;
  int                           actualX, actualY, lastX, lastY, lastGlobalX, lastGlobalY;
  int                           elapsed;
  MpdCom::Status                status;
  MpdCom                        mpdCom;
  MpdCom                        playlistMpdCom;
  PlaylistWindow                plw;
  ConfigDialog                  configDialog;
  WASkin                      * skin;
};

#endif
