#ifndef __PLAYLIST__H
#define __PLAYLIST__H

#include <qstring.h>
#include <vector>

/*! \author     <a href="mailto:mpdc@hokitoki.dds.nl">ing. A.E. van Putten</a>
    \date       01-01-2004
    \brief      Playlist class for storing a playlist.
 */
class Playlist
{
public:

  /*! \author     <a href="mailto:mpdc@hokitoki.dds.nl">ing. A.E. van Putten</a>
      \date       01-01-2004
      \brief      SongInfo class for storing song information.
   */
  class SongInfo
  {
  public:
    inline                      SongInfo(void) : filename(), artist(), album(), title(), songLength(0), songIndex(0) {} //!< Creates an instance of this class

    inline void                 clear(unsigned index)                           { filename = ""; artist = ""; album = ""; title = ""; songLength = 0; songIndex = index; } //!< Cleares the information contained in the class
    inline QString              getSongLength(void)                             { return QString::number(songLength/3600) + ":" + QString::number((songLength / 60) % 60) + ":" + QString::number(songLength % 60); } //!< Returns the length of the song as formated time

    QString                     filename;                                       //!< the filename
    QString                     artist;                                         //!< the artist
    QString                     album;                                          //!< the album
    QString                     title;                                          //!< the title
    int                         songLength;                                     //!< the songlength as unformated time in seconds
    unsigned                    songIndex;                                      //!< the index in the playlists
  };

  typedef std::vector<SongInfo *>::const_iterator iterator;                         //!< Iterator for iterating through the playlist
  inline iterator               begin() const             { return list.begin(); }  //!< Returns a pointer to the beginning of the playlist
  inline iterator               end() const               { return list.end(); }    //!< Returns an invalid value for the playlist which marks the end of the list

  inline Playlist::SongInfo * operator [](unsigned i)           { return list[i]; } //!< Operator for returning a specific element of the playlist

public:
                                Playlist(void);                                                             //!< Creates an instance of this class
  inline void                   addPlaylistItem( SongInfo * item )              { list.push_back( item ); } //!< Adds a song to the playlist
  inline SongInfo *             getPlaylistItem( unsigned index )               { return list[index]; }     //!< Returns a specific item from the playlist
  inline void                   clear()                                         { list.clear(); }           //!< Clears the playlist (does not clear the list on the server)
  inline unsigned               size()                                          { return list.size(); }     //!< Returns the number of songs in the playlist

protected:
  std::vector<SongInfo *>       list;                                           //!< The vector containing the songinformation items
};

#endif
