#ifndef __PLAYLISTMANAGER__
#define __PLAYLISTMANAGER__

#include <qlistview.h>
#include <qlayout.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>

#include "mpdc.h"
#include "mpdcom.h"

/*! Allows to load/enqueue/remove playlists, load and traverse directory structures
    and add one or more files to the playlist.
    \author     <a href="mailto:mpdc@hokitoki.dds.nl">ing. A.E. van Putten</a>
    \date       01-01-2004
    \brief      Manages the playlist.
 */
class PlaylistManager : public QDialog
{
Q_OBJECT
public:
  static void                   showme(MpdCom *, bool = false);                 //!< Creates or reuses a window.

protected:
                                PlaylistManager(MpdCom *);                      //!< Constructor, creates an instance of this class

  virtual void                  closeEvent(QCloseEvent *);                      //!< Stores it's window geometry and closes the window.
  virtual void                  resizeEvent(QResizeEvent *);                    //!< Handles the resize events.

protected slots:
  void                          slotReceiveDirectoryContent(const MpdCom::Filelist &);          //!< Handles retrieved directory contents from MPD.
  void                          slotDirectoryDoubleClicked(QListViewItem *);                    //!< Traverses through directories.
  void                          slotFileDoubleClicked(QListViewItem *);                         //!< Adds a file to the playlist.
  void                          slotPlaylistDoubleClicked(QListViewItem *);                     //!< Loads a playlist.
  void                          slotPlaylistRightClick(QListViewItem *, const QPoint &, int);   //!< Shows a popup menu for the playlists.
  void                          slotDirectoryRightClick(QListViewItem *, const QPoint &, int);  //!< Shows a popup menu for the directories.
  void                          slotAddSelection(void);                                         //!< Adds multiple files to the playlist.

private:
  static PlaylistManager      * me;                                             //!< The actual window.
  QString                       currDir;                                        //!< The current directory.
  MpdCom                      * mpdCom;                                         //!< MpdCom connection.

  QHBoxLayout                   mainLayout, buttonLayout;
  QVBoxLayout                   leftLayout, rightLayout;

  QListView                     lvPlaylists, lvDirectories, lvFiles;
  QPushButton                   btAdd, btUpdate;
  bool                          directPlay;
};

#endif
