#ifndef __PLAYLISTWINDOW__
#define __PLAYLISTWINDOW__

#include <qwidget.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpainter.h>

#include "mpdc.h"
#include "playlist.h"
#include "mpdcom.h"
#include "playlistmanager.h"

class ControlPanel;

/*! \author     <a href="mailto:mpdc@hokitoki.dds.nl">ing. A.E. van Putten</a>
    \date       01-01-2004
    \brief      PlaylistWindow class for displaying and editing the playlist.
 */
class PlaylistWindow : public QWidget
{
Q_OBJECT
private:
  /*! \author     <a href="mailto:mpdc@hokitoki.dds.nl">ing. A.E. van Putten</a>
      \date       01-01-2004
      \brief      PlaylistViewItem class for giving addition properties to the QListViewItem.
   */
  class PlaylistViewItem: public QListViewItem
  {
    public:
      inline                    PlaylistViewItem( QListView * parent, QString label1, QString label2 = QString::null, QString label3 = QString::null, QString label4 = QString::null, QString label5 = QString::null, QString label6 = QString::null, QString label7 = QString::null, QString label8 = QString::null )
                                :QListViewItem(parent,label1,label2,label3,label4,label5,label6,label7,label8),
                                 playing(false) {}

      virtual void              paintCell ( QPainter * p, const QColorGroup & cg, int column, int width, int align );
      inline bool               isPlaying(void)           { return playing; }
      inline void               setPlaying(bool on)       { playing = on; }

    private:
      bool                      playing;
  };

  /*! \author     <a href="mailto:mpdc@hokitoki.dds.nl">ing. A.E. van Putten</a>
      \date       01-01-2004
      \brief      PlaylistView class for allowing addition functionality for the playlist.
   */
  class PlayListView: public QListView
  {
  public:
    inline                      PlayListView(MpdCom * mpdCom, QWidget * parent):QListView(parent), mpdCom(mpdCom), noUpdate(false)  {}
    inline bool                 canUpdate(void)                                 { return !noUpdate; }

  protected:
    virtual void                keyPressEvent(QKeyEvent *);
    virtual void                contentsMousePressEvent(QMouseEvent *);
    virtual void                contentsMouseMoveEvent ( QMouseEvent *);
    virtual void                contentsMouseReleaseEvent(QMouseEvent *);

  private:
    MpdCom                    * mpdCom;
    PlaylistViewItem          * dragging;
    int                         prevMouseY;
    bool                        noUpdate;
  };

public:
                                PlaylistWindow(MpdCom *, ControlPanel *);                       //!< Contructs an instance of this class
  virtual                       ~PlaylistWindow(void);                                          //!< Destruct the instance of this class

  virtual void                  show(void);                                                     //!< Shows the playlist window
  virtual void                  hide(void);                                                     //!< Hides the playlist window

private slots:
  void                          slotPlaylistUpdate( const Playlist & );
  void                          slotDoubleClicked( QListViewItem * );
  void                          slotRefreshPlaylist( void );
  void                          slotFind( void );
  void                          slotReceiveUpdate( const MpdCom::Status & );
  void                          slotAddSong( void );
  void                          slotRemoveSong( void );
  void                          slotClearPlaylist( void );

protected:
  virtual void                  closeEvent(QCloseEvent *);                      //!< Stores it's window position and closes

private:
  void                          markPlaying(PlaylistViewItem *, bool);

  QGridLayout                   mainLayout;
  QHBoxLayout                   topLayout;
  QHBoxLayout                   bottomLayout;
  MpdCom                * const mpdcom;
  ControlPanel          * const controlPanel;
  PlayListView                  songList;
  QLineEdit                     editArtist, editTitle;
  QLabel                        labArtist, labTitle;
  QPushButton                   btAdd, btRemove, btClear, btRefresh;
  MpdCom::Status                status;
//  PlaylistManager             * playlistManager;
  PlaylistViewItem            * currentPlaying;
};

#endif
