#include <iostream>
#include <qbitmap.h>

#include "waskin.h"

#include "controlpanel.h"
#include "gbutton.h"
#include "gtextview.h"
#include "gslider.h"

#include "xpm/main.xpm"
#include "xpm/cbuttons.xpm"
#include "xpm/shufrep.xpm"
#include "xpm/text.xpm"
#include "xpm/volume.xpm"
#include "xpm/balance.xpm"
#include "xpm/playpaus.xpm"
#include "xpm/numbers.xpm"
#include "xpm/posbar.xpm"
#include "xpm/titlebar.xpm"


WASkin::WASkin(ControlPanel *panel)
       :name(""),
        file(""),
        panel(panel),
        imageMain(),
        imageButtons(),
        imageOption(),
        imageText(),
        imageVolume(),
        imageBalance(),
        imagePlayPaus(),
        imageNumbers(),
        imagePosition(),
        imageTitlebar(),
        numbers(0)
{
}

WASkin::WASkin(QString name, QString file, ControlPanel *panel)
       :name(name),
        file(file.latin1()),
        panel(panel),
        imageMain(),
        imageButtons(),
        imageOption(),
        imageText(),
        imageVolume(),
        imageBalance(),
        imagePlayPaus(),
        imageNumbers(),
        imagePosition(),
        imageTitlebar(),
        numbers(0)
{
}

void WASkin::apply(void)
{
  numbers = 11;

  if (strlen(file.getName()) > 0)
  {
    // Load the images
    imageMain     = QPixmap(file.extract("main.bmp"));
    imageButtons  = QPixmap(file.extract("cbuttons.bmp"));
    imageOption   = QPixmap(file.extract("shufrep.bmp"));
    imageText     = QPixmap(file.extract("text.bmp"));
    imageVolume   = QPixmap(file.extract("volume.bmp"));
    imageBalance  = QPixmap(file.extract("balance.bmp"));
    imagePlayPaus = QPixmap(file.extract("playpaus.bmp"));
    imageNumbers  = QPixmap(file.extract("nums_ex.bmp"));
    imagePosition = QPixmap(file.extract("posbar.bmp"));
    imageTitlebar = QPixmap(file.extract("titlebar.bmp"));

    // Secundary
    if (imageBalance.isNull())   imageBalance  = QPixmap(file.extract("volume.bmp"));
    if (imageNumbers.isNull()) { imageNumbers  = QPixmap(file.extract("numbers.bmp")); numbers = 10; }
  }

  // Fallback
  if (imageMain.isNull())     imageMain     = QPixmap((const char **)main_xpm);
  if (imageButtons.isNull())  imageButtons  = QPixmap((const char **)cbuttons_xpm);
  if (imageOption.isNull())   imageOption   = QPixmap((const char **)shufrep_xpm);
  if (imageText.isNull())     imageText     = QPixmap((const char **)text_xpm);
  if (imageVolume.isNull())   imageVolume   = QPixmap((const char **)volume_xpm);
  if (imageBalance.isNull())  imageBalance  = QPixmap((const char **)balance_xpm);
  if (imagePlayPaus.isNull()) imagePlayPaus = QPixmap((const char **)playpaus_xpm);
  if (imageNumbers.isNull())  imageNumbers  = QPixmap((const char **)numbers_xpm);
  if (imagePosition.isNull()) imagePosition = QPixmap((const char **)posbar_xpm);
  if (imageTitlebar.isNull()) imageTitlebar = QPixmap((const char **)titlebar_xpm);

  panel->setSkin(this);
}

void WASkin::unload(void)
{
  // Unload the images
  imageMain     = QPixmap();
  imageButtons  = QPixmap();
  imageOption   = QPixmap();
  imageText     = QPixmap();
  imageVolume   = QPixmap();
  imageBalance  = QPixmap();
  imagePlayPaus = QPixmap();
  imageNumbers  = QPixmap();
  imagePosition = QPixmap();
  imageTitlebar = QPixmap();
}

void WASkin::setMainWidget(QWidget &widget)
{
  #if QT_VERSION >= 300
    widget.setErasePixmap(imageMain);
  #else
    widget.setBackgroundPixmap(imageMain);
  #endif

  QImage image = imageMain.convertToImage();
  if (isTransparent(image.pixel(0,0)) ||
      isTransparent(image.pixel(274,0)) ||
      isTransparent(image.pixel(0,115)) ||
      isTransparent(image.pixel(274,115)))
    widget.setMask(imageMain.createHeuristicMask(true));
  else
    widget.clearMask();
}

bool WASkin::isTransparent(QRgb p)
{
  int r = qRed(p);
  int g = qGreen(p);
  int b = qBlue(p);

  if ((r == 128) || (r == 255)) r = 1; else r = 0;
  if ((g == 128) || (g == 255)) g = 1; else g = 0;
  if ((b == 128) || (b == 255)) b = 1; else b = 0;

  return ((r + g + b) > 0) && ((r + g + b) < 3);
}

void WASkin::setControlButtons(GButton &btPrev, GButton &btPlay, GButton &btPause, GButton &btStop, GButton &btNext, GButton &btEject)
{
  btPrev.move(16, 88);
  btPrev.resize(23, 18);
  btPrev.pmPressed = QPixmap(23, 18);
  btPrev.pmReleased = QPixmap(23, 18);
  bitBlt(&btPrev.pmPressed, 0, 0, &imageButtons, 0, 18, 23, 18);
  bitBlt(&btPrev.pmReleased, 0, 0, &imageButtons, 0, 0, 23, 18);
  btPrev.repaint();

  btPlay.move(39, 88);
  btPlay.resize(23, 18);
  btPlay.pmPressed = QPixmap(23, 18);
  btPlay.pmReleased = QPixmap(23, 18);
  bitBlt(&btPlay.pmPressed, 0, 0, &imageButtons, 23, 18, 23, 18);
  bitBlt(&btPlay.pmReleased, 0, 0, &imageButtons, 23, 0, 23, 18);
  btPlay.repaint();

  btPause.move(62, 88);
  btPause.resize(23, 18);
  btPause.pmPressed = QPixmap(23, 18);
  btPause.pmReleased = QPixmap(23, 18);
  bitBlt(&btPause.pmPressed, 0, 0, &imageButtons, 46, 18, 23, 18);
  bitBlt(&btPause.pmReleased, 0, 0, &imageButtons, 46, 0, 23, 18);
  btPause.repaint();

  btStop.move(85, 88);
  btStop.resize(23, 18);
  btStop.pmPressed = QPixmap(23, 18);
  btStop.pmReleased = QPixmap(23, 18);
  bitBlt(&btStop.pmPressed, 0, 0, &imageButtons, 69, 18, 23, 18);
  bitBlt(&btStop.pmReleased, 0, 0, &imageButtons, 69, 0, 23, 18);
  btStop.repaint();

  btNext.move(108, 88);
  btNext.resize(22, 18);
  btNext.pmPressed = QPixmap(22, 18);
  btNext.pmReleased = QPixmap(22, 18);
  bitBlt(&btNext.pmPressed, 0, 0, &imageButtons, 92, 18, 22, 18);
  bitBlt(&btNext.pmReleased, 0, 0, &imageButtons, 92, 0, 22, 18);
  btNext.repaint();

  btEject.move(136, 89);
  btEject.resize(22, 16);
  btEject.pmPressed = QPixmap(22, 16);
  btEject.pmReleased = QPixmap(22, 16);
  bitBlt(&btEject.pmPressed, 0, 0, &imageButtons, 114, 16, 22, 16);
  bitBlt(&btEject.pmReleased, 0, 0, &imageButtons, 114, 0, 22, 16);
  btEject.repaint();
}

void WASkin::setOptionButtons(GButton &btEq, GButton &btPl, GButton &btShuffle, GButton &btRepeat)
{
  btEq.move(219, 58);
  btEq.resize(23, 12);
  btEq.pmPressed = QPixmap(23, 12);
  btEq.pmReleased = QPixmap(23, 12);
  btEq.pmtPressed = QPixmap(23, 12);
  btEq.pmtReleased = QPixmap(23, 12);
  bitBlt(&btEq.pmPressed, 0, 0, &imageOption, 46, 61, 23, 12);
  bitBlt(&btEq.pmReleased, 0, 0, &imageOption, 0, 61, 23, 12);
  bitBlt(&btEq.pmtPressed, 0, 0, &imageOption, 46, 73, 23, 12);
  bitBlt(&btEq.pmtReleased, 0, 0, &imageOption, 0, 73, 23, 12);
  btEq.repaint();

  btPl.move(241, 58);
  btPl.resize(23, 12);
  btPl.pmPressed = QPixmap(23, 12);
  btPl.pmReleased = QPixmap(23, 12);
  btPl.pmtPressed = QPixmap(23, 12);
  btPl.pmtReleased = QPixmap(23, 12);
  bitBlt(&btPl.pmPressed, 0, 0, &imageOption, 68, 61, 23, 12);
  bitBlt(&btPl.pmReleased, 0, 0, &imageOption, 22, 61, 23, 12);
  bitBlt(&btPl.pmtPressed, 0, 0, &imageOption, 68, 73, 23, 12);
  bitBlt(&btPl.pmtReleased, 0, 0, &imageOption, 22, 73, 23, 12);
  btPl.repaint();

  btShuffle.move(164, 89);
  btShuffle.resize(46, 15);
  btShuffle.pmPressed = QPixmap(46, 15);
  btShuffle.pmReleased = QPixmap(46, 15);
  btShuffle.pmtPressed = QPixmap(46, 15);
  btShuffle.pmtReleased = QPixmap(46, 15);
  bitBlt(&btShuffle.pmPressed, 0, 0, &imageOption, 28, 15, 46, 15);
  bitBlt(&btShuffle.pmReleased, 0, 0, &imageOption, 28, 0, 46, 15);
  bitBlt(&btShuffle.pmtPressed, 0, 0, &imageOption, 28, 45, 46, 15);
  bitBlt(&btShuffle.pmtReleased, 0, 0, &imageOption, 28, 30, 46, 15);
  btShuffle.repaint();

  btRepeat.move(210, 89);
  btRepeat.resize(27, 15);
  btRepeat.pmPressed = QPixmap(27, 15);
  btRepeat.pmReleased = QPixmap(27, 15);
  btRepeat.pmtPressed = QPixmap(27, 15);
  btRepeat.pmtReleased = QPixmap(27, 15);
  bitBlt(&btRepeat.pmPressed, 0, 0, &imageOption, 0, 15, 27, 15);
  bitBlt(&btRepeat.pmReleased, 0, 0, &imageOption, 0, 0, 27, 15);
  bitBlt(&btRepeat.pmtPressed, 0, 0, &imageOption, 0, 45, 27, 15);
  bitBlt(&btRepeat.pmtReleased, 0, 0, &imageOption, 0, 30, 27, 15);
  btRepeat.repaint();
}

void WASkin::setTextView(GTextView &txTitle, GTextView &txKbps, GTextView &txKHz)
{
  txTitle.move(111, 27);
  txTitle.resize(155, 6);
  txKbps.move(111, 43);
  txKbps.resize(15, 6);
  txKHz.move(155, 43);
  txKHz.resize(10, 6);

  for (unsigned i=0; i<31; i++)
  {
    txTitle.pmChar[i] = QPixmap(5, 6);
    bitBlt(&txTitle.pmChar[i], 0, 0, &imageText, i*5, 0, 5, 6);
    txTitle.pmChar[i+31] = QPixmap(5, 6);
    bitBlt(&txTitle.pmChar[i+31], 0, 0, &imageText, i*5, 6, 5, 6);
    txTitle.pmChar[i+62] = QPixmap(5, 6);
    bitBlt(&txTitle.pmChar[i+62], 0, 0, &imageText, i*5, 12, 5, 6);

    txKbps.pmChar[i] = QPixmap(5, 6);
    bitBlt(&txKbps.pmChar[i], 0, 0, &imageText, i*5, 0, 5, 6);
    txKbps.pmChar[i+31] = QPixmap(5, 6);
    bitBlt(&txKbps.pmChar[i+31], 0, 0, &imageText, i*5, 6, 5, 6);
    txKbps.pmChar[i+62] = QPixmap(5, 6);
    bitBlt(&txKbps.pmChar[i+62], 0, 0, &imageText, i*5, 12, 5, 6);

    txKHz.pmChar[i] = QPixmap(5, 6);
    bitBlt(&txKHz.pmChar[i], 0, 0, &imageText, i*5, 0, 5, 6);
    txKHz.pmChar[i+31] = QPixmap(5, 6);
    bitBlt(&txKHz.pmChar[i+31], 0, 0, &imageText, i*5, 6, 5, 6);
    txKHz.pmChar[i+62] = QPixmap(5, 6);
    bitBlt(&txKHz.pmChar[i+62], 0, 0, &imageText, i*5, 12, 5, 6);
  }

  txTitle.repaint();
  txKbps.repaint();
  txKHz.repaint();
}

void WASkin::setSliders(GSlider &slVolume, GSlider &slBalance, GSlider &slPosition)
{
  slVolume.move(107, 57);
  slVolume.resize(68, 13);
  slVolume.pmStage = new QPixmap[28];
  slVolume.stageCount = 28;

  if (true)
  for (unsigned i=0; i<28; i++)
  {
    slVolume.pmStage[i] = QPixmap(68, 13);
    bitBlt(&slVolume.pmStage[i], 0, 0, &imageVolume, 0, i * 15, 68, 13);
  }

  if (imageVolume.size().height() > 430)
  {
    slVolume.pmButtonDown = QPixmap(14, 11);
    slVolume.pmButtonUp = QPixmap(14, 11);
    bitBlt(&slVolume.pmButtonDown, 0, 0, &imageVolume, 0, 422, 14, 11);
    bitBlt(&slVolume.pmButtonUp, 0, 0, &imageVolume, 15, 422, 14, 11);
  }
  else
  {
    slVolume.pmButtonDown = QPixmap();
    slVolume.pmButtonUp = QPixmap();
  }

  slVolume.hoffset = 1;
  debug( "slVolume.currentStage: " << slVolume.currentStage );
  #if QT_VERSION >= 300
    slVolume.setErasePixmap(slVolume.pmStage[slVolume.currentStage]);
  #else
    slVolume.setBackgroundPixmap(slVolume.pmStage[slVolume.currentStage]);
  #endif
  slVolume.repaint();

  slBalance.move(177, 57);
  slBalance.resize(38, 13);
  slBalance.pmStage = new QPixmap[28];
  slBalance.stageCount = 28;

  if (true)
  for (unsigned i=0; i<28; i++)
  {
    slBalance.pmStage[i] = QPixmap(38, 13);
    bitBlt(&slBalance.pmStage[i], 0, 0, &imageBalance, 9, i * 15, 38, 13);
  }

  if (imageVolume.size().height() > 430)
  {
    slBalance.pmButtonDown = QPixmap(14, 11);
    slBalance.pmButtonUp = QPixmap(14, 11);
    bitBlt(&slBalance.pmButtonDown, 0, 0, &imageBalance, 0, 422, 14, 11);
    bitBlt(&slBalance.pmButtonUp, 0, 0, &imageBalance, 15, 422, 14, 11);
  }
  else
  {
    slBalance.pmButtonDown = QPixmap();
    slBalance.pmButtonUp = QPixmap();
  }

  slBalance.hoffset = 1;
  debug( "slBalance.currentStage: " << slBalance.currentStage );
  #if QT_VERSION >= 300
    slBalance.setErasePixmap(slBalance.pmStage[slBalance.currentStage]);
  #else
    slBalance.setBackgroundPixmap(slBalance.pmStage[slBalance.currentStage]);
  #endif
  slBalance.repaint();

  slPosition.move(16, 72);
  slPosition.resize(248, 10);
  slPosition.pmStage = new QPixmap[1];
  slPosition.stageCount = 1;
  slPosition.pmStage[0] = QPixmap(248, 10);
  bitBlt(&slPosition.pmStage[0], 0, 0, &imagePosition, 0, 0, 248, 10);
  slPosition.pmButtonDown = QPixmap(29, 10);
  slPosition.pmButtonUp = QPixmap(29, 10);
  bitBlt(&slPosition.pmButtonDown, 0, 0, &imagePosition, 278, 0, 29, 10);
  bitBlt(&slPosition.pmButtonUp, 0, 0, &imagePosition, 248, 0, 29, 10);

  #if QT_VERSION >= 300
    slPosition.setErasePixmap(slPosition.pmStage[slPosition.currentStage]);
  #else
    slPosition.setBackgroundPixmap(slPosition.pmStage[slPosition.currentStage]);
  #endif
  slPosition.repaint();
}

void WASkin::setTitleButtons(GButton &btMin, GButton &btMax, GButton &btClose)
{
  btMin.move(244, 3);
  btMin.resize(9, 9);
  btMin.pmPressed = QPixmap(9, 9);
  btMin.pmReleased = QPixmap(9, 9);
  bitBlt(&btMin.pmPressed, 0, 0, &imageTitlebar, 9, 9, 9, 9);
  bitBlt(&btMin.pmReleased, 0, 0, &imageTitlebar, 9, 0, 9, 9);
  btMin.repaint();

  btMax.move(254, 3);
  btMax.resize(9, 9);
  btMax.pmPressed = QPixmap(9, 9);
  btMax.pmReleased = QPixmap(9, 9);
  btMax.pmtPressed = QPixmap(9, 9);
  btMax.pmtReleased = QPixmap(9, 9);
  bitBlt(&btMax.pmPressed, 0, 0, &imageTitlebar, 9, 18, 9, 9);
  bitBlt(&btMax.pmReleased, 0, 0, &imageTitlebar, 0, 18, 9, 9);
  bitBlt(&btMax.pmtPressed, 0, 0, &imageTitlebar, 9, 27, 9, 9);
  bitBlt(&btMax.pmtReleased, 0, 0, &imageTitlebar, 0, 27, 9, 9);
  btMax.repaint();

  btClose.move(264, 3);
  btClose.resize(9, 9);
  btClose.pmPressed = QPixmap(9, 9);
  btClose.pmReleased = QPixmap(9, 9);
  bitBlt(&btClose.pmPressed, 0, 0, &imageTitlebar, 18, 9, 9, 9);
  bitBlt(&btClose.pmReleased, 0, 0, &imageTitlebar, 18, 0, 9, 9);
  btClose.repaint();
}

QPixmap & WASkin::getState(int s)
{
  static QPixmap px;
  px = QPixmap(9, 9);

  if (s > 5) s = 5;
  bitBlt(&px, 0, 0, &imagePlayPaus, s * 9, 0, 9, 9);

  return px;
}

QPixmap & WASkin::getNumber(int s)
{
  static QPixmap px;
  px = QPixmap(9, 13);

  if ((unsigned)s > numbers) s = numbers;
  bitBlt(&px, 0, 0, &imageNumbers, s * 9, 0, 9, 13);

  return px;
}
